
#include "HMG.ch"
#include "common.ch"

#DEFINE olFolderCalendar      9
#DEFINE olAppointmentItem     1
#DEFINE CRLF                  CHR(13)+CHR(10)

STATIC oOutlook
STATIC oNameSpace

FUNCTION InitMAPI()
LOCAL lRet       := .F.
LOCAL bSaveError
LOCAL oError

   bSaveError := ERRORBLOCK( { | oError | MyErrorHandler( oError ) } )
   BEGIN SEQUENCE
      oOutlook := CreateObject( "Outlook.Application" )
      lRet := .T.
   RECOVER
   END SEQUENCE
   ERRORBLOCK( bSaveError )
   IF EMPTY( oOutLook )
      msgInfo( "Outlook not installed !", "Error CreateObject()" )
      RETURN lRet
   ENDIF

   bSaveError := ERRORBLOCK( { | oError | MyErrorHandler( oError ) } )
   BEGIN SEQUENCE
      oNameSpace := oOutlook:GetNamespace( "MAPI" )
      lRet := .T.
   RECOVER
   END SEQUENCE
   ERRORBLOCK( bSaveError )
   IF EMPTY( oNameSpace )
      msgInfo( "can't get Namespace MAPI", "Error MAPI" )
      ExitMAPI()
      RETURN lRet
   ENDIF

RETURN lRet


FUNCTION ExitMAPI()
LOCAL lRet       := .F.
LOCAL bSaveError
LOCAL oError

   bSaveError := ERRORBLOCK( { | oError | MyErrorHandler( oError ) } )
   BEGIN SEQUENCE
      oNameSpace := NIL
      oOutlook:Quit()
      oOutlook := NIL
      lRet := .T.
   RECOVER USING oError
   END SEQUENCE
   ERRORBLOCK( bSaveError )

RETURN lRet

FUNCTION MyErrorHandler( oError )                                     
   BREAK
RETURN .F.

FUNCTION AddNewItem( cSubject, cMemo, dDate, cTime, nMinute, cCategories, cLocation, lAllDayEvent, lReminderSet )
LOCAL oAppointment, oItem
LOCAL cStringCreDate := ""
LOCAL cLASTDATE      := ""
LOCAL lRet           := .F.
LOCAL nCount         := 0
LOCAL bSaveError
LOCAL oError

   DEFAULT cCategories TO ""
   DEFAULT cLocation TO ""
   DEFAULT lAllDayEvent TO .F.
   DEFAULT lReminderSet TO .F.

   IF SP_lUseMapi() = .T.
      IF InitMAPI()

         bSaveError := ERRORBLOCK( { | oError | MyErrorHandler( oError ) } )
         BEGIN SEQUENCE
            oAppointment := oNameSpace:getDefaultFolder( olFolderCalendar )
         RECOVER USING oError
            ERRORBLOCK( bSaveError )
            oAppointment := NIL
         END SEQUENCE
         ERRORBLOCK( bSaveError )
         IF NIL = oAppointment
            MsgInfo( "can not get Items", "RPC-Server is not avaiable" )
            RETURN .F.
         ENDIF

         oItem := oOutlook:CreateItem( olAppointmentItem )
         oItem:Start 	:= hb_SToT( DTOS( dDate ) + StripTime( cTime ) )
         oItem:Duration := nMinute           
         oItem:Subject 	:= TRIM( cSubject )
         oItem:Body 	:= cMemo

         oItem:Categories  := cCategories
         oItem:Location    := cLocation
         oItem:AllDayEvent := lAllDayEvent
         oItem:ReminderSet := lReminderSet
         //   oItem:ReminderTime := nBeforeTime

         // save
         oItem:save()

         // need to wait ???
         //
         nCount := 0
         DO WHILE .T.
            cStringCreDate := Read_CreationTime( oItem , .F.)
            cLASTDATE := Read_LastModificationTime( oItem, .F. )

            IF EMPTY( cStringCreDate )
               nCount ++
               IF nCount > 100
                  MsgInfo( "Error MAPI Read_CreationTime()" )
                  lRet := .F.
                  EXIT
               ELSE
                  SLEEP( 2 )
               ENDIF
            ELSE
               lRet := .T.
               EXIT
            ENDIF
         ENDDO

         ExitMAPI()

      ENDIF
   ENDIF

   bSaveError := ERRORBLOCK( { | oError | MyErrorHandler( oError ) } )
   BEGIN SEQUENCE
      oOutlook:Quit()
      oOutlook := NIL
   RECOVER USING oError
   END SEQUENCE
   ERRORBLOCK( bSaveError )

RETURN lRet


STATIC FUNCTION Read_CreationTime( oItem, lReplace )
LOCAL oError
LOCAL cStringStart := ""
LOCAL cDateStart
LOCAL cTimeStart
LOCAL bSaveError   := ERRORBLOCK( { | oError | MyErrorHandler( oError ) } )

   DEFAULT lReplace TO .T.

   BEGIN SEQUENCE
      //  MUST !!! else "corrupt" !
      cStringStart := hb_TToC( oItem:CreationTime )

      cDateStart := SUBSTR( cStringStart, 1, 10 )
      cTimeStart := SUBSTR( cStringStart, 12, 12 )                    // YYYY.MM.DDHHMMSS

      cStringStart := DTOS( CTOD( cDateStart ) ) + StripTime( cTimeStart )

      IF lReplace = .T.
         REPLACE OUTLOOK->CREDATE WITH cStringStart
      ENDIF
   RECOVER USING oError
   END SEQUENCE
   ERRORBLOCK( bSaveError )
RETURN cStringStart


STATIC FUNCTION Read_LastModificationTime( oItem, lReplace )
LOCAL oError
LOCAL cStringStart := ""
LOCAL cDateStart
LOCAL cTimeStart
LOCAL bSaveError   := ERRORBLOCK( { | oError | MyErrorHandler( oError ) } )

   DEFAULT lReplace TO .T.

   BEGIN SEQUENCE
      //  MUST !!! else "corrupt" !
      cStringStart := hb_TToC( oItem:LastModificationTime )

      cDateStart := SUBSTR( cStringStart, 1, 10 )
      cTimeStart := SUBSTR( cStringStart, 12, 12 )                    // YYYY.MM.DDHHMMSS

      cStringStart := DTOS( CTOD( cDateStart ) ) + StripTime( cTimeStart )

      IF lReplace = .T.
         REPLACE OUTLOOK->LASTDATE WITH cStringStart
      ENDIF
   RECOVER USING oError
   END SEQUENCE
   ERRORBLOCK( bSaveError )
RETURN cStringStart

FUNCTION SP_lUseMapi( xValue )
STATIC lUseMapi := .T.
   IF PCOUNT() > 0
      lUseMapi := xValue
   ENDIF
RETURN lUseMapi

*
* eof
*
